#! /usr/bin/env python3

"""This script is used to upload container image scan reports to AWS S3.

Args:
    report_path: The path to the local directory containing the scan reports.
    bucket_name: The name of the S3 bucket to send the reports to.

Example:
    s3_uploader.py -rp "reports/folder" -bn "bucket_name"

Raises:
    ClientError: If the AWS client is unable to fulfill the request.
"""

import argparse
from botocore.exceptions import ClientError
import datetime
import logging
import os

import boto3

image_list = os.getenv("SCAN_IMAGE_LIST")

if image_list is None or image_list == "":
    print("Nothing to upload to S3. Skipping ..")
    exit(0)
PRODUCTION_SCAN_PATH = "production_scan"

parser = argparse.ArgumentParser(description='CIS - S3 uploader')
parser.add_argument('-rp', '--report_path',
                    help='Path to the local directory containing the scan reports',
                    required=True, type=str)
parser.add_argument('-bn', '--bucket_name',
                    help='Name of the S3 bucket to send the reports to',
                    required=True, type=str)

args = vars(parser.parse_args())

report_path = args['report_path'] if args['report_path'] else os.environ.get("REP_PATH_CLI", ".")
bucket_name = args['bucket_name'] if args['bucket_name'] else 'cb-container-image-scans'

logging.basicConfig(filename=f'{os.environ.get("REP_PATH_CLI", ".")}/s3_uploader.log',
                    format='%(asctime)s,%(msecs)d %(levelname)-8s [%(filename)s:%(lineno)d] %(message)s',
                    datefmt='%Y-%m-%d:%H:%M:%S',
                    level=logging.DEBUG)

today = datetime.date.today()


def upload_to_s3(path_to_file, b_key, fname):
    filename = f'{today}__{fname}'

    try:
        s3_client.upload_file(
            Filename=path_to_file, Bucket=bucket_name, Key=f'{b_key}/{filename}')
        logging.info(f"Uploading: {b_key}/{filename}")
    except ClientError as e:
        logging.error(e)

    if b_key in ["psec", "html"] and os.environ.get("STORE_AS_PRODUCTION_SCAN") == "true":
        # This is a scan run for production images. Also store the artifacts
        # under PRODUCTION_SCAN_PATH in s3. This path will be crawled from the
        # compliance pipeline to generate report on the production images
        # (running state)
        try:
            s3_client.upload_file(
                Filename=path_to_file, Bucket=bucket_name, Key=f'{PRODUCTION_SCAN_PATH}/{b_key}/{filename}')
            logging.info(f"Uploading: {PRODUCTION_SCAN_PATH}/{b_key}/{filename}")
        except ClientError as e:
            logging.error(e)


print('Uploading scan results to S3..')

boto_session = boto3.Session(aws_access_key_id=os.environ.get('AWS_SECPROD_KEY_ID'),
                             aws_secret_access_key=os.environ.get('AWS_SECPROD_SECRET'),
                             region_name=os.environ.get('AWS_SECPROD_CIS_BUCKET_REGION'))

sts_client = boto_session.client('sts')

try:
    sts_client.get_caller_identity()
    logging.debug('The AWS credentials are valid. Proceeding..')
except ClientError as e:
    logging.error(e)

image_list = image_list.split()
s3_client = boto_session.client('s3')

files_list = [
    f for f in os.listdir(report_path) if
    (f.startswith('cbctl') or f.startswith('twistcli') or f.startswith('psec')) and
    (f.endswith('.json') or f.endswith('.html')) and
    os.path.isfile(os.path.join(report_path, f))
]

for img in image_list:
    img_name = img.split('/')[-1].split(':')[0]
    b_key = ''

    for fname in files_list:
        if img_name in fname:
            scanner = fname.split('_', 1)[0]

            if fname.endswith('.html'):
                b_key = 'html'
            else:
                b_key = scanner

            upload_to_s3(os.path.join(report_path, fname), b_key, fname)

print('Scan results successfully uploaded to S3.')
